@@ -1,15 +1,11 @@
 package Text::Xslate::PP::Opcode;
-use Mouse;
+use Any::Moose;
 extends qw(Text::Xslate::PP::State);
 
-our $VERSION = '1.9999_01';
-
 use Carp ();
 use Scalar::Util ();
 
-use Text::Xslate::PP;
 use Text::Xslate::PP::Const;
-use Text::Xslate::PP::Method;
 use Text::Xslate::Util qw(
     p neat
     mark_raw unmark_raw html_escape uri_escape
@@ -20,6 +16,7 @@ use constant _DUMP_PP => scalar($DEBUG =~ /\b dump=pp \b/xms);
 
 no warnings 'recursion';
 
+require Text::Xslate::PP;
 if(!Text::Xslate::PP::_PP_ERROR_VERBOSE()) {
     our @CARP_NOT = qw(
         Text::Xslate
@@ -71,25 +68,6 @@ sub op_localize_s {
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
-sub op_localize_vars {
-    my($st) = @_;
-    my $new_vars = $st->{sa};
-    my $old_vars = $st->vars;
-
-    if(ref($new_vars) ne 'HASH') {
-        $st->warn(undef, "Variable map must be a HASH reference");
-    }
-
-    push @{ $st->{local_stack} }, bless sub {
-            $st->vars($old_vars);
-            return;
-        }, 'Text::Xslate::PP::Guard';
-
-    $st->vars($new_vars);
-
-    goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
-}
-
 sub op_push {
     push @{ $_[0]->{ SP }->[ -1 ] }, $_[0]->{sa};
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
@@ -107,11 +85,6 @@ sub op_nil {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_vars {
-    $_[0]->{sa} = $_[0]->{vars};
-
-    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
-}
 
 sub op_literal {
     $_[0]->{sa} = $_[0]->op_arg;
@@ -150,7 +123,24 @@ sub op_fetch_field_s {
 
 sub op_print {
     my($st) = @_;
-    $st->print($st->{sa});
+    my $sv = $st->{sa};
+
+    if ( ref( $sv ) eq TXt_RAW ) {
+        if(defined ${$sv}) {
+            $st->{ output } .= ${$sv};
+        }
+        else {
+            $st->warn(undef, "Use of nil to print" );
+        }
+    }
+    elsif ( defined $sv ) {
+        $sv =~ s/($Text::Xslate::PP::html_metachars)/$Text::Xslate::PP::html_escape{$1}/xmsgeo;
+        $st->{ output } .= $sv;
+    }
+    else {
+        $st->warn( undef, "Use of nil to print" );
+    }
+
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
 }
 
@@ -174,29 +164,27 @@ sub op_print_raw_s {
 
 
 sub op_include {
-    my($st) = @_;
-    my $child = Text::Xslate::PP::tx_load_template( $st->engine, $st->{sa}, 1 );
-    $st->push_frame('include', undef);
-    my $output = Text::Xslate::PP::tx_execute( $child, $st->{vars} );
-    $st->pop_frame(0);
-    $st->{output} .= $output;
-    goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
-}
+    my $st = Text::Xslate::PP::tx_load_template( $_[0]->engine, $_[0]->{sa}, 1 );
+    $_[0]->{ output } .= Text::Xslate::PP::tx_execute( $st, $_[0]->{vars} );
 
-sub op_find_file {
-    $_[0]->{sa} = eval { $_[0]->engine->find_file($_[0]->{sa}); 1 };
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_suffix {
-    $_[0]->{sa} = $_[0]->engine->{suffix};
-    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
-}
 
 sub op_for_start {
     my($st) = @_;
+    my $ar = $st->{sa};
     my $id = $st->op_arg;
-    my $ar = Text::Xslate::PP::tx_check_itr_ar($st, $st->{sa});
+
+    unless ( $ar and ref $ar eq 'ARRAY' ) {
+        if ( defined $ar ) {
+            $st->error( undef, "Iterator variables must be an ARRAY reference, not %s", neat( $ar ) );
+        }
+        else {
+            $st->warn( undef, "Use of nil to iterate" );
+        }
+        $ar = [];
+    }
 
     #tx_access_lvar( $st, $id + TXfor_ITEM, undef );
     tx_access_lvar( $st, $id + TXfor_ITER, -1 );
@@ -220,8 +208,6 @@ sub op_for_iter {
             goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
         }
         else {
-            # finish the loop
-            $st->{sa} = ( $i > 0 ); # for 'for-else' block
             tx_access_lvar( $st, $id + TXfor_ITEM,  undef );
             tx_access_lvar( $st, $id + TXfor_ITER,  undef );
             tx_access_lvar( $st, $id + TXfor_ARRAY, undef );
@@ -382,23 +368,18 @@ sub op_builtin_html_escape {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_builtin_uri_escape {
+sub op_builtin_uri {
     $_[0]->{sa} = uri_escape($_[0]->{sa});
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
 sub op_builtin_is_array_ref {
-    $_[0]->{sa} = Text::Xslate::Util::is_array_ref($_[0]->{sa});
+    $_[0]->{sa} = ref($_[0]->{sa}) eq 'ARRAY';
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
 sub op_builtin_is_hash_ref {
-    $_[0]->{sa} = Text::Xslate::Util::is_hash_ref($_[0]->{sa});
-    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
-}
-
-sub op_is_code_ref {
-    $_[0]->{sa} = Text::Xslate::Util::is_code_ref($_[0]->{sa});
+    $_[0]->{sa} = ref($_[0]->{sa}) eq 'HASH';
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
@@ -480,9 +461,11 @@ sub tx_macro_enter {
         return;
     }
 
-    my $cframe = $st->push_frame($name, $retaddr);
+    my $cframe = Text::Xslate::PP::tx_push_frame( $st );
 
+    $cframe->[ TXframe_RETADDR ] = $retaddr;
     $cframe->[ TXframe_OUTPUT ]  = $st->{ output };
+    $cframe->[ TXframe_NAME ]    = $name;
 
     $st->{ output } = '';
 
@@ -509,14 +492,16 @@ sub tx_macro_enter {
 
 sub op_macro_end {
     my($st) = @_;
+    my $frames   = $st->frame;
+    my $oldframe = $frames->[ $st->current_frame ];
+    my $cframe   = $frames->[ $st->current_frame( $st->current_frame - 1 ) ]; # pop frame
 
-    my $top = $st->frame->[ $st->current_frame ];
-    printf STDERR "%stx_macro_end(%s)]\n", ' ' x $st->current_frame - 1, $top->[ TXframe_NAME ] if _DUMP_PP;
+    print STDERR " " x $st->current_frame, "tx_macro_end($oldframe->[ TXframe_NAME ])\n" if _DUMP_PP;
 
     $st->{sa} = mark_raw( $st->{ output } );
-    $st->pop_frame(1);
 
-    $st->{ pc } = $top->[ TXframe_RETADDR ];
+    $st->{ output } = $oldframe->[ TXframe_OUTPUT ];
+    $st->{ pc }     = $oldframe->[ TXframe_RETADDR ];
     goto $st->{ code }->[ $st->{ pc } ]->{ exec_code };
 }
 
@@ -535,6 +520,7 @@ sub op_funcall {
 
 sub op_methodcall_s {
     my($st) = @_;
+    require Text::Xslate::PP::Method;
     $st->{sa} = Text::Xslate::PP::Method::tx_methodcall(
         $st, undef, $st->op_arg, @{ pop @{ $st->{SP} } });
     goto $st->{ code }->[ ++$st->{ pc } ]->{ exec_code };
@@ -552,11 +538,6 @@ sub op_make_hash {
     goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
 }
 
-sub op_merge_hash {
-    $_[0]->{sa} = Text::Xslate::Util::merge_hash($_[0]->{sa}, $_[0]->{sb});
-    goto $_[0]->{ code }->[ ++$_[0]->{ pc } ]->{ exec_code };
-}
-
 
 sub op_enter {
     push @{$_[0]->{save_local_stack} ||= []}, delete $_[0]->{local_stack};
@@ -581,7 +562,7 @@ sub op_end {
     $st->{ pc } = $st->code_len;
 
     if($st->current_frame != 0) {
-        #Carp::croak("Oops: broken stack frame:" .  p($st->frame));
+        Carp::croak("Oops: broken stack frame:" .  p($st->frame));
     }
     return;
 }
@@ -635,7 +616,7 @@ sub proccall {
     }
 }
 
-no Mouse;
+no Any::Moose;
 __PACKAGE__->meta->make_immutable();
 __END__
 
